@@ -7,10 +7,13 @@
 //! using a custom type for the constant `1`. As computing an identity column w.r.t. the current
 //! basis is equivalent to just inverting the basis explicitly.
 use std::iter;
+use std::iter::Once;
 
 use relp_num::One;
 
 use crate::algorithm::two_phase::matrix_provider::column::Column;
+use crate::algorithm::two_phase::matrix_provider::column::ColumnIterator;
+use crate::data::linear_algebra::SparseTuple;
 
 /// Identity columns are needed for artificial matrices.
 ///
@@ -44,7 +47,7 @@ impl IdentityColumn {
 
 impl Column for IdentityColumn {
     type F = One;
-    type Iter<'a> = std::iter::Once<(usize, &'a Self::F)>;
+    type Iter<'a> = impl ColumnIterator<'a, F=Self::F>;
 
     fn iter(&self) -> Self::Iter<'_> {
         iter::once((self.index, &ONE))
@@ -58,3 +61,12 @@ impl Column for IdentityColumn {
         }.to_string()
     }
 }
+
+impl IntoIterator for IdentityColumn {
+    type Item = SparseTuple<One>;
+    type IntoIter = Once<Self::Item>;
+
+    fn into_iter(self) -> Self::IntoIter {
+        iter::once((self.index, One))
+    }
+}
