@@ -4,7 +4,7 @@ use crate::algorithm::two_phase::matrix_provider::column::Column;
 use crate::algorithm::two_phase::matrix_provider::MatrixProvider;
 use crate::algorithm::two_phase::strategy::pivot_rule::PivotRule;
 use crate::algorithm::two_phase::tableau::{debug_assert_in_basic_feasible_solution_state, Tableau};
-use crate::algorithm::two_phase::tableau::inverse_maintenance::{ColumnComputationInfo, InverseMaintener, ops as im_ops};
+use crate::algorithm::two_phase::tableau::inverse_maintenance::{ColumnComputationInfo, InverseMaintainer, ops as im_ops};
 use crate::algorithm::two_phase::tableau::kind::non_artificial::NonArtificial;
 
 /// Reduces the cost of the basic feasible solution to the minimum.
@@ -19,19 +19,25 @@ use crate::algorithm::two_phase::tableau::kind::non_artificial::NonArtificial;
 ///
 /// An `OptimizationResult` indicating whether or not the problem has a finite optimum. It cannot be
 /// infeasible, as a feasible solution is needed to start using this method.
-pub fn primal<IM, MP, PR>(
-    tableau: &mut Tableau<IM, NonArtificial<MP>>,
+pub fn primal<'provider, IM, MP, PR>(
+    tableau: &mut Tableau<IM, NonArtificial<'provider, MP>>,
 ) -> OptimizationResult<IM::F>
 where
-    IM: InverseMaintener<F: im_ops::FieldHR + im_ops::Column<<MP::Column as Column>::F>>,
-    for<'r> IM::F: im_ops::Cost<MP::Cost<'r>>,
+    IM: InverseMaintainer<F:
+        im_ops::FieldHR +
+        im_ops::Column<<MP::Column as Column>::F> +
+        im_ops::Cost<MP::Cost<'provider>> +
+    >,
     MP: MatrixProvider,
     PR: PivotRule<IM::F>,
 {
     let mut rule = PR::new(tableau);
 
     loop {
-        debug_assert_in_basic_feasible_solution_state(tableau);
+        debug_assert!({
+            debug_assert_in_basic_feasible_solution_state(&tableau);
+            true
+        });
 
         match rule.select_primal_pivot_column(tableau) {
             Some((pivot_column_index, cost)) => {
