@@ -144,11 +144,11 @@ where
         column
     }
 
-    fn left_multiply_by_basis_inverse<'a, G: 'a + ColumnNumber, I: ColumnIterator<'a, G>>(
-        &self, column: I,
+    fn left_multiply_by_basis_inverse<'a, I: ColumnIterator<'a>>(
+        &'a self, column: I,
     ) -> Self::ColumnComputationInfo
     where
-        Self::F: ops::Column<G>,
+        Self::F: ops::Column<I::F>,
     {
         let tuples = (0..self.m())
             .map(|i| self.generate_element(i, column.clone()))
@@ -159,11 +159,11 @@ where
         SparseVector::new(tuples, self.m())
     }
 
-    fn right_multiply_by_basis_inverse<'a, G: 'a + ColumnNumber, I: ColumnIterator<'a, G>>(
+    fn right_multiply_by_basis_inverse<'a, I: ColumnIterator<'a>>(
         &self, mut row: I,
     ) -> SparseVector<Self::F, Self::F>
     where
-        Self::F: ops::Column<G>,
+        Self::F: ops::Column<I::F>,
     {
         let (index, factor) = row.next().unwrap();
         let items = self.rows[index].iter().map(|(j, value)| (j, value * factor)).collect();
@@ -176,17 +176,17 @@ where
         total
     }
 
-    fn generate_element<'a, G: 'a + ColumnNumber, I: ColumnIterator<'a, G>>(
-        &self,
+    fn generate_element<'a, I: ColumnIterator<'a>>(
+        &'a self,
         i: usize,
         original_column: I,
     ) -> Option<Self::F>
     where
-        Self::F: ops::Column<G>,
+        Self::F: ops::Column<I::F>,
     {
         debug_assert!(i < self.m());
 
-        let element = self.rows[i].sparse_inner_product::<Self::F, _, _>(original_column);
+        let element = self.rows[i].sparse_inner_product(original_column);
         if element.is_not_zero() {
             Some(element)
         } else {
@@ -291,8 +291,8 @@ mod test {
     #[test]
     fn invert_identity() {
         let columns = [
-            matrix_data::Column::Slack([(0, RB!(1))], []),
-            matrix_data::Column::Slack([(1, RB!(1))], []),
+            matrix_data::Column::Slack((0, RB!(1))),
+            matrix_data::Column::Slack((1, RB!(1))),
         ];
 
         let result = BasisInverseRows::<RationalBig>::invert(columns.into_iter());
@@ -308,8 +308,8 @@ mod test {
     #[test]
     fn invert_non_identity() {
         let columns = [
-            matrix_data::Column::TwoSlack([(0, RB!(1)), (1, RB!(1))], []),
-            matrix_data::Column::Slack([(1, RB!(1))], []),
+            matrix_data::Column::TwoSlack((0, RB!(1)), (1, RB!(1))),
+            matrix_data::Column::Slack((1, RB!(1))),
         ];
 
         let result = BasisInverseRows::<RationalBig>::invert(columns.into_iter());
