@@ -106,10 +106,15 @@ where
         }
 
         // We collected row-major but need column major, so transpose
-        let mut upper_triangular = vec![Vec::new(); m];
+        let mut upper_triangular = vec![Vec::new(); m - 1];
+        let mut upper_diagonal = Vec::with_capacity(m);
         for (i, row) in rows.into_iter().enumerate() {
-            for (j, value) in row {
-                upper_triangular[j].push((i, value));
+            let mut iter = row.into_iter();
+            let (index, diagonal) = iter.next().unwrap();
+            debug_assert_eq!(index, i);
+            upper_diagonal.push(diagonal);
+            for (j, value) in iter {
+                upper_triangular[j - 1].push((i, value));
             }
         }
         let mut lower_triangular = vec![Vec::new(); m - 1];
@@ -127,12 +132,12 @@ where
         let mut column_permutation = FullPermutation::new(column_permutation);
         column_permutation.invert();
 
-
         Self {
             row_permutation,
             column_permutation,
             lower_triangular,
             upper_triangular,
+            upper_diagonal,
             updates: Vec::new(),
         }
     }
@@ -300,15 +305,16 @@ fn count_non_zeros<T>(rows: &[Vec<(usize, T)>]) -> (Vec<usize>, Vec<usize>) {
 
 #[cfg(test)]
 mod test {
-    use relp_num::RB;
+    use relp_num::{RB, R8, RationalBig, NonZero, Rational8};
 
-    use crate::algorithm::two_phase::matrix_provider::column::Column;
+    use crate::algorithm::two_phase::matrix_provider::column::{Column, SparseSliceIterator, DenseSliceIterator};
     use crate::algorithm::two_phase::matrix_provider::column::identity::IdentityColumn;
     use crate::algorithm::two_phase::tableau::inverse_maintenance::carry::BasisInverse;
     use crate::algorithm::two_phase::tableau::inverse_maintenance::carry::lower_upper::LUDecomposition;
     use crate::algorithm::two_phase::tableau::inverse_maintenance::carry::lower_upper::permutation::FullPermutation;
     use crate::algorithm::two_phase::tableau::inverse_maintenance::ColumnComputationInfo;
     use crate::data::linear_algebra::vector::{SparseVector, Vector};
+    use std::collections::VecDeque;
 
     #[test]
     fn identity_2() {
@@ -318,7 +324,8 @@ mod test {
             row_permutation: FullPermutation::identity(2),
             column_permutation: FullPermutation::identity(2),
             lower_triangular: vec![vec![]],
-            upper_triangular: vec![vec![(0, RB!(1))], vec![(1, RB!(1))]],
+            upper_triangular: vec![vec![]],
+            upper_diagonal: vec![RB!(1), RB!(1)],
             updates: vec![],
         };
 
@@ -333,7 +340,8 @@ mod test {
             row_permutation: FullPermutation::identity(3),
             column_permutation: FullPermutation::identity(3),
             lower_triangular: vec![vec![], vec![]],
-            upper_triangular: vec![vec![(0, RB!(1))], vec![(1, RB!(1))], vec![(2, RB!(1))]],
+            upper_triangular: vec![vec![], vec![]],
+            upper_diagonal: vec![RB!(1), RB!(1), RB!(1)],
             updates: vec![],
         };
 
@@ -348,7 +356,8 @@ mod test {
             row_permutation: FullPermutation::identity(2),
             column_permutation: FullPermutation::identity(2),
             lower_triangular: vec![vec![]],
-            upper_triangular: vec![vec![(0, RB!(1))], vec![(0, RB!(1)), (1, RB!(1))]],
+            upper_triangular: vec![vec![(0, RB!(1))]],
+            upper_diagonal: vec![RB!(1), RB!(1)],
             updates: vec![],
         };
 
@@ -363,7 +372,8 @@ mod test {
             row_permutation: FullPermutation::identity(2),
             column_permutation: FullPermutation::identity(2),
             lower_triangular: vec![vec![(1, RB!(1))]],
-            upper_triangular: vec![vec![(0, RB!(1))], vec![(1, RB!(1))]],
+            upper_triangular: vec![vec![]],
+            upper_diagonal: vec![RB!(1), RB!(1)],
             updates: vec![],
         };
 
@@ -378,7 +388,8 @@ mod test {
             row_permutation: FullPermutation::new(vec![1, 0]),
             column_permutation: FullPermutation::identity(2),
             lower_triangular: vec![vec![(1, RB!(1))]],
-            upper_triangular: vec![vec![(0, RB!(1))], vec![(1, RB!(1))]],
+            upper_triangular: vec![vec![]],
+            upper_diagonal: vec![RB!(1), RB!(1)],
             updates: vec![],
         };
 
@@ -393,7 +404,8 @@ mod test {
             row_permutation: FullPermutation::identity(2),
             column_permutation: FullPermutation::identity(2),
             lower_triangular: vec![vec![(1, RB!(3, 2))]],
-            upper_triangular: vec![vec![(0, RB!(4))], vec![(0, RB!(3)), (1, RB!(-3, 2))]],
+            upper_triangular: vec![vec![(0, RB!(3))]],
+            upper_diagonal: vec![RB!(4), RB!(-3, 2)],
             updates: vec![],
         };
 
@@ -408,7 +420,8 @@ mod test {
             row_permutation: FullPermutation::identity(2),
             column_permutation: FullPermutation::identity(2),
             lower_triangular: vec![vec![(1, RB!(-1))]],
-            upper_triangular: vec![vec![(0, RB!(-1))], vec![(0, RB!(3, 2)), (1, RB!(1, 2))]],
+            upper_triangular: vec![vec![(0, RB!(3, 2))]],
+            upper_diagonal: vec![RB!(-1), RB!(1, 2)],
             updates: vec![],
         };
 
@@ -424,6 +437,219 @@ mod test {
         );
     }
 
+    pub fn to_columns<const M: usize>(rows: &[[i32; M]; M]) -> VecDeque<Vec<(usize, Rational8)>> {
+        let mut columns = vec![vec![]; M].into_iter().collect::<VecDeque<_>>();
+
+        for (i, row) in rows.into_iter().enumerate() {
+            for (j, v) in row.into_iter().enumerate() {
+                if v.is_not_zero() {
+                    columns[j].push((i, R8!(*v)));
+                }
+            }
+        }
+
+        columns
+    }
+
+    fn test_matrix<const M: usize>(rows: [[i32; M]; M]) {
+        let columns = to_columns(&rows);
+        let result = LUDecomposition::<RationalBig>::rows(
+            rows.iter().map(|row| {
+                row.iter().enumerate()
+                    .filter(|(_, v)| v.is_not_zero())
+                    .map(|(i, v)| (i, v.into()))
+                    .collect()
+            }).collect()
+        );
+        for (j, column) in columns.iter().enumerate() {
+            assert_eq!(
+                result.left_multiply_by_basis_inverse(SparseSliceIterator::new(column)).into_column(),
+                SparseVector::standard_basis_vector(j, M),
+                "{}", j,
+            );
+        }
+        for (j, row) in rows.into_iter().enumerate() {
+            assert_eq!(
+                result.right_multiply_by_basis_inverse(DenseSliceIterator::new(&row)),
+                SparseVector::standard_basis_vector(j, M),
+                "{}", j,
+            );
+        }
+    }
+
+    #[test]
+    fn test_3x3() {
+        test_matrix([
+            [ 2,  3,  0],
+            [ 5,  0, 11],
+            [23, 29,  0],
+        ]);
+    }
+
+    #[test]
+    fn test_4x4_1() {
+        test_matrix([
+            [ 2,  3,  0,  5],
+            [ 5,  0, 11, 13],
+            [23, 29,  0, 57],
+            [31, 37, 41,  0],
+        ]);
+    }
+
+    #[test]
+    fn test_4x4_2() {
+        test_matrix([
+            [-101,    0,    0,   -5],
+            [-110,  -81,    0,    0],
+            [   0,    0,    1, -111],
+            [   0,   93,   69,    0],
+        ]);
+    }
+
+    #[test]
+    fn test_4x4_3() {
+        test_matrix([
+            [  0,   0, -84, 122],
+            [  0,   9,   0,   0],
+            [-39, 115,   0,  57],
+            [  0, -12, 121,   0],
+        ]);
+    }
+
+    #[test]
+    fn test_5x5_banded() {
+        test_matrix([
+            [2,  3,  0,  0,  0],
+            [5,  7, 11,  0,  0],
+            [0, 29, 13, 57,  0],
+            [0,  0, 41, 17,  0],
+            [0,  0,  0, 53, 51],
+        ]);
+    }
+
+    #[test]
+    fn test_5x5_1() {
+        test_matrix([
+            [29, 23,  0, 19, 0],
+            [ 0,  0, 17, 13, 0],
+            [ 0,  0,  7,  0, 0],
+            [ 5,  0,  0,  3, 0],
+            [ 0,  0,  0,  0, 2],
+        ]);
+    }
+
+    #[test]
+    fn test_5x5_2() {
+        test_matrix([
+            [29, 23,  0, 19, 0],
+            [ 0,  0, 17, 13, 0],
+            [ 0, 11,  7,  0, 0],
+            [ 5,  0,  0,  3, 0],
+            [ 0,  0,  0,  0, 2],
+        ]);
+    }
+
+    #[test]
+    fn test_5x5_3() {
+        test_matrix([
+            [ 2,  3,  0,  5,  7],
+            [ 5,  0, 11, 13, 17],
+            [23, 29,  0, 57, 59],
+            [31, 37, 41,  0,  0],
+            [43,  0, 47, 53, 51],
+        ]);
+    }
+
+    #[test]
+    fn test_5x5_4() {
+        test_matrix([
+            [ 2,  3,  0,  5,  7],
+            [ 5,  0, 11, 13, 17],
+            [23, 29,  0, 57, 59],
+            [31, 37, 41,  0,  0],
+            [43,  0, 47, 53, 51],
+        ]);
+    }
+
+    #[test]
+    fn test_5x5_5() {
+        test_matrix([
+            [   0,   54,   43,    0,   84],
+            [   4,    0,    0,    0,    0],
+            [   0, -111,  -27,    0,  -86],
+            [  -6,    0,    0,   17,  -62],
+            [-109,    0,    0,    0, -104],
+        ]);
+    }
+
+    #[test]
+    fn test_5x5_6() {
+        test_matrix([
+            [ -71, -124,    0,    0, -108],
+            [   0,   66, -121,  -74,  -53],
+            [   0,  104,    0,    0,    0],
+            [   0,   55,    0,    1,   -3],
+            [  93,    0,    0,    0,  104],
+        ]);
+    }
+
+    #[test]
+    fn test_6x6_1() {
+        test_matrix([
+            [   0,    0,    0,  -25,    0,    0],
+            [ -15,   79,    0,    0,    0,    0],
+            [   0,    0,    0,    0,    0,   14],
+            [   0,    0,    0, -114,  -61,    0],
+            [   0,    0,  109,    0,    0, -126],
+            [  46,    0,    0,   50,   21,    0],
+        ]);
+    }
+
+    #[test]
+    fn test_6x6_2() {
+        test_matrix([
+            [   0,    0,  -26,  -68,   84,    0],
+            [-125,   43,    0,    0,    0,  -63],
+            [   0,    0,    1,   90,    0,    0],
+            [   0,  -81,    0,    0,    0,    0],
+            [ -15,    0,    0,  -81,    0,    0],
+            [   0,  -12,    0,    0,    0,    1],
+        ]);
+    }
+
+    #[test]
+    fn test_10x10_1() {
+        test_matrix([
+            [   0,    0,    0,    0,    0,    0,  -60,    0,  -10,    0],
+            [   0,    0,    0,    0,    0,    0,    0,  -84,    0,    0],
+            [   0, -105,    0,    0,    0,    0,    0,    0,    0,    0],
+            [   0,    0,    0,  -25,    0,    0,    0,    0,  116,    0],
+            [   0,    0,    0,    0,  -18,    0,    0,    0,    0,    0],
+            [   0,    0,    0,  -72,    0,    0,    0,    0,    0,    0],
+            [   0,    0,   16,   48,    0,    0,    0,    0,    0,    0],
+            [ -57,    0,    0,  -88,  107,    0,    0,    0,    0,    0],
+            [-122, -108,    0,    0,    0,   91,    0,    0, -127,    0],
+            [   0,   85,    0,    0,  106,    0,    0,    0,    0, -121],
+        ]);
+    }
+
+    #[test]
+    fn test_11x11_1() {
+        test_matrix([
+            [   0,    0,    0,    0,    0,    0,    0,    0,    0,  -13,    0],
+            [   0,    0,    0,    0,    0,    0,    0,    0,    0,    0, -122],
+            [   0,    0,    0,    0,    0,  102,   82,    0,    0,    0,   13],
+            [   0,    0,    0,    0,    0, -107,  -39,    0,    0,    0,    0],
+            [   0,    0,    0,    0,    0,    0,  -39,   48, -113,    0,    0],
+            [  24,    0,    0,    0,    0,    0,    0,    0,    0,  -93, -120],
+            [-111,    0,    0,  -81,    0,    0,    0,    0,    0,    0,    0],
+            [   0,    0,    0,    0,    0,   82,    0,    0,   76,    0,    0],
+            [   0,    0,  -51,    0,    0,    0,  126,    0,    0,    0, -105],
+            [   0,  118,    0,    0,    0,    0,    0,    0,    0,    0,   27],
+            [   0,    0,  120,    0,  -31,    0,    0,    0,    0,    0,    0],
+        ]);
+    }
+
     mod subtract_multiple_of_column_from_other_column {
         use relp_num::R32;
 
