@@ -16,7 +16,7 @@ use crate::algorithm::two_phase::matrix_provider::column::identity::IdentityColu
 use crate::algorithm::two_phase::matrix_provider::filter::Filtered;
 use crate::algorithm::two_phase::matrix_provider::MatrixProvider;
 use crate::algorithm::two_phase::tableau::BasisChangeComputationInfo;
-use crate::algorithm::two_phase::tableau::inverse_maintenance::{ColumnComputationInfo, InverseMaintener, ops};
+use crate::algorithm::two_phase::tableau::inverse_maintenance::{ColumnComputationInfo, InverseMaintainer, ops};
 use crate::algorithm::two_phase::tableau::kind::Kind;
 use crate::data::linear_algebra::SparseTuple;
 use crate::data::linear_algebra::traits::Element;
@@ -118,23 +118,23 @@ pub trait BasisInverse: Display {
     /// A `SparseVector<T>` of length `m`.
     /// TODO(ENHANCEMENT): Drop the `OrderedColumn` trait bound once it is possible to specialize on
     ///  it, some implementations don't need it.
-    fn left_multiply_by_basis_inverse<'a, G: 'a + ColumnNumber, I: ColumnIterator<'a, G>>(
-        &self,
+    fn left_multiply_by_basis_inverse<'a, I: ColumnIterator<'a>>(
+        &'a self,
         column: I,
     ) -> Self::ColumnComputationInfo
     where
-        Self::F: ops::Column<G>,
+        Self::F: ops::Column<I::F>,
     ;
 
     /// Multiply a row vector with the basis inverse matrix from the right.
     ///
     /// This is the same as multiplying with the transpose of the basis inverse matrix from the left
     /// when you consider the input vector a column vector: B^-T v = (v^T B^-1)^T.
-    fn right_multiply_by_basis_inverse<'a, G: 'a + ColumnNumber, I: ColumnIterator<'a, G>>(
+    fn right_multiply_by_basis_inverse<'a, I: ColumnIterator<'a>>(
         &self, row: I,
     ) -> SparseVector<Self::F, Self::F>
     where
-        Self::F: ops::Column<G>
+        Self::F: ops::Column<I::F>
     ;
 
     /// Generate a single element in the tableau with respect to the current basis.
@@ -145,13 +145,13 @@ pub trait BasisInverse: Display {
     /// * `original_column`: Column with respect to the original basis.
     /// TODO(ENHANCEMENT): Drop the `OrderedColumn` trait bound once it is possible to specialize on
     ///  it.
-    fn generate_element<'a, G: 'a + ColumnNumber, I: ColumnIterator<'a, G>>(
-        &self,
+    fn generate_element<'a, I: ColumnIterator<'a>>(
+        &'a self,
         i: usize,
         original_column: I,
     ) -> Option<Self::F>
     where
-        Self::F: ops::Column<G>,
+        Self::F: ops::Column<I::F>,
     ;
 
     /// Whether this basis inverse should be recomputed.
@@ -335,9 +335,13 @@ where
     /// # Note
     ///
     /// This method requires a normalized pivot element.
-    fn update_minus_pi_and_obj(&mut self, pivot_row_index: usize, relative_cost: F) {
-        let basis_inverse_row = self.basis_inverse.basis_inverse_row(pivot_row_index);
-        for (column_index, value) in SparseSliceIterator::new(&basis_inverse_row) {
+    fn update_minus_pi_and_obj(
+        &mut self,
+        pivot_row_index: usize,
+        relative_cost: F,
+        basis_inverse_row: &SparseVector<F, F>,
+    ) {
+        for (column_index, value) in SparseSliceIterator::new(basis_inverse_row) {
             self.minus_pi[column_index] -= &relative_cost * value;
         }
 
@@ -359,7 +363,7 @@ where
     }
 }
 
-impl<F, BI> InverseMaintener for Carry<F, BI>
+impl<F, BI> InverseMaintainer for Carry<F, BI>
 where
     F: ops::Field + ops::FieldHR + Signed,
     BI: BasisInverse<F=F>,
@@ -569,7 +573,7 @@ where
         debug_assert_eq!(column_computation_info.column().len(), self.m());
 
         let iter = SparseSliceIterator::new(column_computation_info.column());
-        let work_vector = self.basis_inverse.right_multiply_by_basis_inverse::<Self::F, _>(iter);
+        let work_vector = self.basis_inverse.right_multiply_by_basis_inverse(iter);
 
         // The order of these calls matters: the first of the two normalizes the pivot row
         self.update_b(pivot_row_index, column_computation_info.column());
@@ -586,10 +590,8 @@ where
             self.basis_inverse.change_basis(pivot_row_index, column_computation_info)
         };
 
-        self.update_minus_pi_and_obj(pivot_row_index, relative_cost);
-
-        let column = IdentityColumn::new(pivot_row_index);
-        let basis_inverse_row = self.basis_inverse.right_multiply_by_basis_inverse(column.iter()).into_column();
+        let basis_inverse_row = self.basis_inverse.basis_inverse_row(pivot_row_index);
+        self.update_minus_pi_and_obj(pivot_row_index, relative_cost, &basis_inverse_row);
 
         BasisChangeComputationInfo {
             pivot_row_index,
@@ -601,12 +603,11 @@ where
         }
     }
 
-    fn cost_difference<G, C: Column<F=G>>(&self, original_column: &C) -> Self::F
+    fn cost_difference<C: Column>(&self, original_column: &C) -> Self::F
     where
-        Self::F: ops::Column<G>,
-        G: Display + Debug,
+        Self::F: ops::Column<C::F>,
     {
-        self.minus_pi.sparse_inner_product::<Self::F, _, _>(original_column.iter())
+        self.minus_pi.sparse_inner_product(original_column.iter())
     }
 
     fn generate_column<C: Column>(
@@ -664,7 +665,7 @@ where
     }
 }
 
-impl<F, BI> InverseMaintener for Carry<F, BI>
+impl<F, BI> InverseMaintainer for Carry<F, BI>
 where
     F: ops::Field + ops::FieldHR + Signed,
     BI: BasisInverse<F=F> + RemoveBasisPart,
